@@ -7,10 +7,19 @@ import re
 from copy import copy
 
 from powerline.lib.unicode import unicode
-from powerline.lint.markedjson.error import echoerr, DelayedEchoErr
+from powerline.lint.markedjson.error import echoerr, DelayedEchoErr, NON_PRINTABLE_STR
 from powerline.lint.selfcheck import havemarks
 
 
+NON_PRINTABLE_RE = re.compile(
+	NON_PRINTABLE_STR.translate({
+		ord('\t'): None,
+		ord('\n'): None,
+		0x0085: None,
+	})
+)
+
+
 class Spec(object):
 	'''Class that describes some JSON value
 
@@ -154,7 +163,7 @@ class Spec(object):
 		'''Define message which will be used when unknown key was found
 
 		“Unknown” is a key that was not provided at the initialization and via 
-		:py:meth:`Spec.update` and did not match any ``keyfunc`` proided via 
+		:py:meth:`Spec.update` and did not match any ``keyfunc`` provided via 
 		:py:meth:`Spec.unknown_spec`.
 
 		:param msgfunc:
@@ -255,7 +264,7 @@ class Spec(object):
 		:param func msg_func:
 			Callable like ``msg_func`` from :py:meth:`Spec.check_func`. Should 
 			accept one problematic item and is not used for :py:class:`Spec` 
-			object indicies in ``item_func`` method.
+			object indices in ``item_func`` method.
 
 		:return: proceed, hadproblem.
 		'''
@@ -300,11 +309,16 @@ class Spec(object):
 		``self.specs[start:end]`` is matched by the given value.
 		'''
 		havemarks(value)
-		new_echoerr = DelayedEchoErr(echoerr)
+		new_echoerr = DelayedEchoErr(
+			echoerr,
+			'One of the either variants failed. Messages from the first variant:',
+			'messages from the next variant:'
+		)
 
 		hadproblem = False
 		for spec in self.specs[start:end]:
 			proceed, hadproblem = spec.match(value, value.mark, data, context, new_echoerr)
+			new_echoerr.next_variant()
 			if not proceed:
 				break
 			if not hadproblem:
@@ -342,6 +356,26 @@ class Spec(object):
 				return False, hadproblem
 		return True, hadproblem
 
+	def check_printable(self, value, context_mark, data, context, echoerr, _):
+		'''Check that given unicode string contains only printable characters
+		'''
+		hadproblem = False
+		for match in NON_PRINTABLE_RE.finditer(value):
+			hadproblem = True
+			echoerr(
+				context=self.cmsg.format(key=context.key),
+				context_mark=value.mark,
+				problem='found not printable character U+{0:04x} in a configuration string'.format(
+					ord(match.group(0))),
+				problem_mark=value.mark.advance_string(match.start() + 1)
+			)
+		return True, hadproblem
+
+	def printable(self, *args):
+		self.type(unicode)
+		self.checks.append(('check_printable', args))
+		return self
+
 	def type(self, *args):
 		'''Describe value that has one of the types given in arguments
 
@@ -481,7 +515,7 @@ class Spec(object):
 			optional. Each element in this list describes element with the same 
 			index in the checked value. Check out :py:meth:`Spec.check_tuple` 
 			for more details, but note that there list of specifications is 
-			replaced with start and end indicies in ``self.specs``.
+			replaced with start and end indices in ``self.specs``.
 
 		:return: self.
 		'''
@@ -497,7 +531,8 @@ class Spec(object):
 		if max_len == min_len:
 			self.len('eq', len(specs))
 		else:
-			self.len('ge', min_len)
+			if min_len > 0:
+				self.len('ge', min_len)
 			self.len('le', max_len)
 
 		start = len(self.specs)
@@ -598,7 +633,7 @@ class Spec(object):
 
 		Check out :py:meth:`Spec.check_either` method documentation for more 
 		details, but note that there a list of specs was replaced by start and 
-		end indicies in ``self.specs``.
+		end indices in ``self.specs``.
 
 		:return: self.
 		'''
